<?php
namespace App\EventSubscriber;

use Symfony\Component\HttpKernel\Event\GetResponseEvent;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\Routing\RouterInterface;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class CheckServiceSubscriber implements EventSubscriberInterface
{
    private $defaultLocale;
    private $router;

    public function __construct(RouterInterface $router)
    {
        $this->defaultLocale = 'de-DE';
        $this->router = $router;
    }

    public function onKernelRequest(GetResponseEvent $event)
    {
        $valid = false;
        $request = $event->getRequest();
        if (!$event->isMasterRequest()) {
            return;
        }
        $user = $request->getSession()->get('_user');
        if ($user == null) {
            return;
        }
        $basePath = $request->getBasePath();
        $pathInfo = $request->getPathInfo();
        $hg = new \Homegear\Homegear;
        $hg->setUserPrivileges("$user");
        $frontend = $hg->checkServiceAccess('ui');
        $backend = $hg->checkServiceAccess('admin-ui');
        $nodeBlue = $hg->checkServiceAccess('node-blue');
        if ($basePath == '/ui') {
            if (stristr($pathInfo,'/admin')) {
                if ($backend === true) {
                    return;
                }
                if ($frontend === true) {
                    // redirect to frontend
                    $response = new RedirectResponse($this->router->generate('index'));
                }
                if ($nodeBlue === true) {
                    // redirect to node-blue
                    $response = new RedirectResponse('/node-blue');
                }
            } else {
                if ($frontend === true) {
                    return;
                }
                if ($backend === true) {
                    // redirect to backend
                    $response = new RedirectResponse($this->router->generate('admin.dashboard'));
                }
                if ($nodeBlue === true) {
                    // redirect to node-blue
                    $response = new RedirectResponse('/node-blue');
                }
            }
        }
        if ($basePath == '/admin') {
            if ($backend === true) {
                return;
            }
            if ($frontend === true) {
                // redirect to frontend
                $response = new RedirectResponse('/ui');
            }
            if ($nodeBlue === true) {
                // redirect to node-blue
                $response = new RedirectResponse('/node-blue');
            }
        }
        $response->send();
    }

    public static function getSubscribedEvents()
    {
        return array(
            // must be registered before (i.e. with a higher priority than) the default Locale listener
            KernelEvents::REQUEST => array(array('onKernelRequest', 20)),
        );
    }
}