<?php
Namespace App\Model\Admin;

use Symfony\Component\Translation\TranslatorInterface;
Use App\Utils\Utils;

/**
 * Provides admin functions for Homegear service management
 *
 */
class HomegearBackup
{
    public $hg;
    private $utils;
    private $translator;

    public function __construct(TranslatorInterface $translator, Utils $utils)
    {
        $this->hg = new \Homegear\Homegear;
        $this->utils = $utils;
        $this->translator = $translator;
    }

    /**
     * Delete old backups
     * 
     * @return none
     */
    public function removeOld()
    {
        exec("rm -f /tmp/*_homegear-backup.tar.gz");
    }

    /**
     * Create a backup
     * 
     * @return json
     */
    public function create()
    {
        $id = $this->hg->managementCreateBackup();
        $result = $this->utils->getCommandStatus($id);
        if ($result['exitCode'] == 0) {
            return ['result' => 'success', 'backup' => $result['metadata']['filename']];
        }
        return ['result' => 'error'];
    }

    /**
     * Restore a backup
     * 
     * @return json
     */
    public function restore($file = null)
    {
        if ($file === null) { return ['result' => 'error']; }
        $restore_db = false;
        $current_db = false;
        $filename = $file->getClientOriginalName();
        rename($file->getPathname(), "/tmp/$filename");
        if ($this->utils->checkOss() === false) {
            // temporary extract backup in order to check db-version
            exec('mkdir /tmp/version_test; tar -zxvf /tmp/'.$filename.' -C /tmp/version_test');
            if (file_exists('/tmp/version_test/var/lib/homegear/db.sql')) {
                $restore_db = '/tmp/version_test/var/lib/homegear/db.sql';
            } elseif (file_exists('/tmp/version_test/data/homegear-data/db/db.sql')) {
                $restore_db = '/tmp/version_test/data/homegear-data/db/db.sql';
            }
            if (file_exists('/var/lib/homegear/db.sql')) {
                $current_db = '/var/lib/homegear/db.sql';
            } elseif (file_exists('/data/homegear-data/db/db.sql')) {
                $current_db = '/data/homegear-data/db/db.sql';
            }
            $backup_version = (trim(shell_exec('echo "SELECT stringValue FROM homegearVariables WHERE variableIndex=0;" |sqlite3 '.$restore_db)));
            $current_version = (trim(shell_exec('echo "SELECT stringValue FROM homegearVariables WHERE variableIndex=0;" |sqlite3 '.$current_db)));
            // remove extracted backup again
            exec('rm -rf /tmp/version_test');
        } else {
            $current_version = 0;
            $backup_version = 0;
        }
        if (version_compare($current_version,$backup_version) >= 0) {
            // all save, backup version is smaller or equal the current version
            try {
                $this->hg->managementRestoreBackup("/tmp/$filename");
                return ['result' => 'success'];
            } catch(\Homegear\HomegearException $e) {
                return ['result' => 'error'];
            }
        } else {
            return ['result' => 'error', 'message' => $this->translator->trans("backups.status_update_first",array(),'admin')];
        }
    }
}