<?php
Namespace App\Model\Admin;

use Symfony\Component\Translation\TranslatorInterface;
Use App\Utils\Utils;

/**
 * Provides admin functions for Homegear update management
 *
 */
class HomegearUpdate
{
    public $hg;
    private $utils;
    private $translator;
    private $componentUpdates = 'Updates';
    private $hold = ['linux-base', 'linux-dtb-next-sunxi', 'linux-image-next-sunxi', 'linux-u-boot-orangepipcplus-next', 'linux-dtb-current-sunxi64', 'linux-image-current-sunxi64', 'linux-u-boot-orangepi3-current'];

    public function __construct(TranslatorInterface $translator, Utils $utils)
    {
        $this->hg = new \Homegear\Homegear;
        $this->utils = $utils;
        $this->translator = $translator;
    }

    public function getLastUpdateCheck()
    {
        return $this->hg->getData($this->componentUpdates,'lastUpdateCheck');
    }

    /**
     * Manually check for updates
     * 
     * @return array()
     */
    public function check()
    {
        if ($this->utils->checkManagementRunning(false)) {
            # clear state
            $this->hg->deleteData($this->componentUpdates,'UpdateAvailable_System');
            $this->hg->deleteData($this->componentUpdates,'UpdateAvailable_Homegear');
            $this->hg->deleteData($this->componentUpdates,'UpdateAvailable_Homegear_packages');
            $this->hg->setData($this->componentUpdates,'lastUpdateCheck',time());
            $updateAvailable = false;
            // check if homegear-smartha-management is running & smarthaAptUpdate is available
            // may be removed after next homegear upgrade ...
            $fromSmartha = false;
            if ($this->utils->checkSmarthaManagementRunning(false) && call_user_func("Homegear\\Homegear::system.listMethods", "smarthaAptUpdate")) {
                $id = $this->hg->smarthaAptUpdate();
                $fromSmartha = true;
            } else {
                $id = $this->hg->managementAptUpdate();
            }
            //$id = $this->hg->managementAptUpdate();
            #$result = $this->utils->getCommandStatus($id);
            $result = $this->utils->waitAptFinished($fromSmartha);
            #if ($result['exitCode'] == 0) {
            if ($result === true) {
                #$updates['System'] = $this->hg->managementSystemUpdateAvailable();
                # workaround: managementSystemUpdateAvailable will last > 15 seconds on some systems!
                $updates['System'] = 1;
                // this is needed because managementHomegearUpdateAvailable() currently checks for updates of the homegear main-package only!
                $packages = $this->_getUpdateList('homegear');
                $updates['Homegear'] = ((isset($packages) && is_array($packages) && count($packages) > 0) ? true : false);
                foreach ($updates as $type => $value) {
                    if ($value && $value != '') {
                        if ($type == 'Homegear') {
                            $currentVersion = str_replace('Homegear ', '', $this->hg->getVersion());
                            // homegear update ... get all updateable packages w/ versions
                            if (isset($packages) && is_array($packages) && count($packages) > 0) {
                                $modulesValid = true;
                                $dependencies = true;
                                /*if (array_key_exists('homegear', $packages)) {
                                    // ok, homegear should be updated ... lets check if ALL installed modules are within the package list, too
                                    if ($this->_checkModules($packages) === false) {
                                        // there is something wrong with the modules, so don't display available update for homegear!
                                        continue;
                                    }
                                }*/
                                // simulate update
                                $missingPackages = $this->_simulateUpdate($packages);
                                if (isset($missingPackages) && is_array($missingPackages) && count($missingPackages) > 0) {
                                    // we have packages which could not be installed, let's check dependencies to see if there are older versions to update
                                    if (!array_key_exists('homegear',$missingPackages) && array_key_exists('homegear',$packages)) {
                                        // if homegear is not in the list of not updatable packages, the new homegear-version is the target version!
                                        $version = $packages['homegear']['update'];
                                    } else {
                                        $version = $currentVersion;
                                        // NEVER update libhomegear-ipc and libhomegear-node, homegear-nodes-core & homegear-nodes-extra if homegear itself will not be updated, otherwise the whole installation may fail!
                                        if (array_key_exists('libhomegear-ipc',$packages)) {
                                            unset($packages['libhomegear-ipc']);
                                        }
                                        if (array_key_exists('libhomegear-node',$packages)) {
                                            unset($packages['libhomegear-node']);
                                        }
                                        if (array_key_exists('homegear-nodes-core',$packages)) {
                                            unset($packages['homegear-nodes-core']);
                                        }
                                        if (array_key_exists('homegear-nodes-extra',$packages)) {
                                            unset($packages['homegear-nodes-extra']);
                                        }
                                    }
                                    $missingPackages = $this->_checkDependencies($version,$missingPackages);
                                    #dump($missingPackages);
                                    foreach ($packages as $package => $version) {
                                        if (array_key_exists($package,$missingPackages)) {
                                            if ($missingPackages[$package]['update'] == $missingPackages[$package]['current']) {
                                                unset($packages[$package]);
                                            } else {
                                                $packages[$package] = $missingPackages[$package];
                                            }
                                        }
                                    }
                                } elseif (!is_array($missingPackages) && $missingPackages === false) {
                                    // something has gone wrong, seems that packages have been uninstalled during simulation
                                    $packages = [];
                                }
                            }
                            if (isset($packages) && is_array($packages) && count($packages) > 0) {
                                // We really have some updates!
                                $this->hg->setData($this->componentUpdates,'UpdateAvailable_'.$type,true);
                                $this->hg->setData($this->componentUpdates,'UpdateAvailable_'.$type.'_packages',$packages);
                                $updateAvailable = true;
                            } else {
                                // No (safe) updates now!
                                $this->hg->deleteData($this->componentUpdates,'UpdateAvailable_'.$type);
                                $this->hg->deleteData($this->componentUpdates,'UpdateAvailable_'.$type.'_packages');
                            }
                        } else {
                            // unfortunately 'apt list --upgradable' will output held back packages, too, so we have to sort em out manually
                            // get a list of packages to update
                            $systemUpdate = false;
                            $systemPackages = $this->_getUpdateList('-v homegear');
                            if (isset($systemPackages) && is_array($systemPackages) && count($systemPackages) > 0) {
                                // check if package is held back
                                foreach ($systemPackages as $systemPackage => $value) {
                                    if (in_array($systemPackage, $this->hold)) { 
                                        unset($systemPackages[$systemPackage]);
                                    }
                                }
                            }
                            // check if we still have upgradable packages
                            if (isset($systemPackages) && is_array($systemPackages) && count($systemPackages) > 0) {
                                // We really have some updates!
                                $this->hg->setData($this->componentUpdates,'UpdateAvailable_'.$type,true);
                                $this->hg->setData($this->componentUpdates,'UpdateAvailable_'.$type.'_packages',$systemPackages);
                                $updateAvailable = true;
                            } else {
                                // No (safe) updates now!
                                $this->hg->deleteData($this->componentUpdates,'UpdateAvailable_'.$type);
                                $this->hg->deleteData($this->componentUpdates,'UpdateAvailable_'.$type.'_packages');
                            }
                        }
                    } else {
                        $this->hg->deleteData($this->componentUpdates,'UpdateAvailable_'.$type);
                        $this->hg->deleteData($this->componentUpdates,'UpdateAvailable_'.$type.'_packages');
                    }
                }
                return ['result' => 'success', 'msg' => (!$updateAvailable ? $this->translator->trans("updates.notfound",array(),'admin') : $this->translator->trans("updates.found",array(),'admin') )];
            }
            return ['result' => 'error', 'msg' => $this->translator->trans('updates.error_checking',array(),'admin')];
        }
        return ['result' => 'error', 'msg' => $this->translator->trans('management.not_running',array(),'admin')];
    }

    /**
     * List all current updates
     * 
     * @return array
     */
    public function list()
    {
        // check if an update is running
        if ($this->utils->checkUpdateRunning('_sys-update-running') === true || $this->utils->checkUpdateRunning('_os-update-running') === true) {
            $os = -1;
            $system = -1;
        } else {
            $os = $this->hg->getData($this->componentUpdates,'UpdateAvailable_System');
            $system = $this->hg->getData($this->componentUpdates,'UpdateAvailable_Homegear');
        }
        return ['system' => $system, 'os' => $os];
    }

    /**
     * Update packages
     * 
     * @return json
     */
    public function update($type = null)
    {
        if ($type === null) { return ['result' => 'error', 'msg' => 'Update-type missing']; }
        if ($type == 'sys') {
            if ($this->utils->checkManagementRunning(false)) {
                // get package data from database
                $fileList = '';
                $files = $this->hg->getData($this->componentUpdates,'UpdateAvailable_Homegear_packages');
                if (is_array($files) && count($files) >0) {
                    foreach ($files as $key => $value) {
                        $fileList .= "$key=".$value['update']." ";
                    }
                }
                if ($fileList != '') {
                    if ($this->utils->checkSmarthaManagementRunning(false)) {
                        $id = $this->hg->smarthaAptUpgradeOverPhp(trim($fileList));
                    } else {
                        $id = $this->hg->managementAptUpgrade(intval(1));
                    }
                    $this->hg->deleteData($this->componentUpdates,'UpdateAvailable_UI');
                    $this->hg->deleteData($this->componentUpdates,'UpdateAvailable_DeviceDefinitions');
                    $this->hg->deleteData($this->componentUpdates,'UpdateAvailable_Nodes');
                    $this->hg->deleteData($this->componentUpdates,'UpdateAvailable_Homegear');
                    $this->hg->deleteData($this->componentUpdates,'UpdateAvailable_Homegear_packages');
                    $this->hg->setData($this->componentUpdates,'UpdateRunning',$id);
                    $this->utils->setOled("Update in progress...");
                    return ['result' => 'success', 'type' => $type, 'id' => $id];
                }
            }
        } elseif ($type == 'os') {
            if ($this->utils->checkManagementRunning(false)) {
                // get package data from database
                $fileList = '';
                $files = $this->hg->getData($this->componentUpdates,'UpdateAvailable_System_packages');
                if (is_array($files) && count($files) >0) {
                    foreach ($files as $key => $value) {
                        $fileList .= "$key=".$value['update']." ";
                    }
                }
                if ($fileList != '') {
                    if ($this->utils->checkSmarthaManagementRunning(false)) {
                        $id = $this->hg->smarthaAptUpgradeOverPhp(trim($fileList));
                    } else {
                        $id = $this->hg->managementAptUpgrade(intval(0));
                    }
                    //$this->utils->saveToSession(['_os-update-running' => $id]);
                    $this->hg->deleteData($this->componentUpdates,'UpdateAvailable_System_packages');
                    $this->hg->setData($this->componentUpdates,'UpdateRunning',$id);
                    $this->utils->setOled("Update in progress...");
                    return ['result' => 'success', 'type' => $type, 'id' => $id];
                }
            }
        }
        return ['result' => 'error', 'msg' => 'Error getting update'];
    }

    private function _getUpdateList($package = false)
    {
        if ($package === false) { return; }
        $packages = array();
        $result = explode("\n",shell_exec('apt list --upgradable 2>/dev/null | grep '.$package));
        if (isset($result) && is_array($result)) {
            foreach ($result as $line) {
                $tmp = explode(" ", $line);
                if (is_array($tmp) && array_key_exists(1,$tmp)) {
                    $packages[explode("/", $tmp[0])[0]]['update'] = $tmp[1];
                    $packages[explode("/", $tmp[0])[0]]['current'] = str_replace(']','',$tmp[5]);
                }
            }
        }
        return $packages;
    }

    private function _checkModules($list = false)
    {
        if ($list === false) { return false; }
        $myModules = $this->hg->listModules();
        foreach ($myModules as $module) {
            if ($module['FAMILY_NAME'] == 'Miscellaneous') { continue; }
            // get package name of the module
            $result = explode(':',shell_exec('dpkg -S /var/lib/homegear/modules/'.$module['FILENAME']));
            if (isset($result) && is_array($result)) {
                if (!array_key_exists($result[0], $list)) {
                    // if one installed package is NOT within the update-list, the whole update may fail
                    return false;
                }
            } else {
                // uups, no package found?
                return false;
            }
        }
        return true;
    }

    private function _simulateUpdate($packages = false)
    {
        if ($packages === false) { return false; }
        $success = array();
        $removed = array();
        $result = explode("\n",shell_exec('apt-get -s dist-upgrade |grep "Remv " |grep homegear'));
        foreach ($result as $line) {
            if (isset($line) && $line != '') {
                $line = preg_replace('/^Remv /','',$line);
                $line = explode(':',preg_replace('/^(.*) \[(.*)\].*/','${1}:${2}',$line));
                $removed[$line[0]]['update'] = $line[1];
            }
        }
        $result = explode("\n",shell_exec('apt-get -s dist-upgrade |grep "Conf " |grep homegear'));
        foreach ($result as $line) {
            if (isset($line) && $line != '') {
                $line = preg_replace('/^Conf /','',$line);
                $line = explode(':',preg_replace('/^(.*) \((\d+.\d.\d+-\d+).*/','${1}:${2}',$line));
                #$line = explode(":",preg_replace('/^.*(homegear.*) \((\d.\d.\d-\d+).*/', '${1}:${2}', $line));
                if (is_array($line) && array_key_exists(0,$line) && substr($line[0], 0, 4) === 'Conf') {
                    $line = explode(":",preg_replace('/^.* (.*-homegear) \((\d+.\d+.\d+-\d+).*/', '${1}:${2}', $line[0]));
                }
                $success[$line[0]]['update'] = $line[1];
            }
        }
        // check if packages have been removed during dry-run ... this could break the whole system and should NEVER EVER happen when updating with ui!
        if (is_array($removed) && count($removed) > 0) {
            return false;
        }
        // return packages which could not be installed during dry-run
        return array_diff_key($packages,$success);
    }

    private function _checkDependencies($version = false, $list = false)
    {
        if ($version === false) { return false; }
        if ($list === false) {
            // we've to no list to check, so get all installed packages for homegear
            $list = $this->_getUpdateList('homegear');
        }
        foreach ($list as $package => $v) {
            $dependencyOK = false;
            $installedVersion = $list[$package]['current'];
            if ($package == 'homegear' || $package == 'libhomegear-base' || $package == 'homegear-nodes-core' || $package == 'homegear-nodes-extra') { 
                $list[$package]['update'] = $version;
                continue;
            }
            $result = explode("\n",shell_exec('apt-cache madison '.$package));
            foreach ($result as $line) {
                if ($line != '') {
                    $line = explode('|',$line);
                    if ($installedVersion == trim($line[1])) {
                        // we have reached the installed version, no need to go further ...
                        $dependencyOK = true;
                        $list[$package]['update'] = $installedVersion;
                        break;
                    }
                    #var_dump('apt-cache show '.$package.'='.trim($line[1]).' |grep Depends');
                    $dependency = shell_exec('apt-cache show '.$package.'='.trim($line[1]).' |grep Depends');
                    if (isset($dependency) && $dependency != null && $dependency != '') {
                        $neededVersion = preg_replace('/^.* homegear \((.* \d.\d.\d-\d+)\), .*\n$/', '${1}', $dependency);
                        if (strpos($neededVersion,"Depends") !== false) {
                            $neededVersion = preg_replace('/^.*, libhomegear-base \((.* \d.\d.\d-\d+)\), .*\n$/', '${1}', $dependency);
                        }
                        if ($neededVersion != '' && strpos($neededVersion,"Depends") === false) {
                            $op = explode(' ',$neededVersion);
                            list($ver,$revision) = explode('-', $op[1]);
                            $current = explode('-', $version);
                            if ($ver == $current[0]) {
                                if (trim($op[0]) == "=") {
                                    if (intval($current[1]) == intval($revision)) {
                                        $dependencyOK = true;
                                        #var_dump("$package: ".$op[0]." $ver $revision");
                                        $list[$package]['update'] = trim($line[1]);
                                        break;
                                    }
                                } elseif (trim($op[0]) == ">=") {
                                    if (intval($current[1]) >= intval($revision)) {
                                        $dependencyOK = true;
                                        #var_dump("$package: ".$op[0]." $ver $revision");
                                        $list[$package]['update'] = trim($line[1]);
                                        break;
                                    }
                                } elseif (trim($op[0]) == "<=") {
                                    if (intval($current[1]) <= intval($revision)) {
                                        $dependencyOK = true;
                                        #var_dump("$package: ".$op[0]." $ver $revision");
                                        $list[$package]['update'] = trim($line[1]);
                                        break;
                                    }
                                }
                            }
                        } else {
                            // no homegear-dependencies found ...
                            $dependencyOK = true;
                            break;
                        }
                    } else {
                        // no dependencies found at all ...
                        $dependencyOK = true;
                        break;
                    }
                }
            }
            if ($dependencyOK === false) {
                // we have a unmet dependency, that could break the whole update!
                return [];
            }
        }
        return $list;
    }    
}