<?php
Namespace App\Model;

use Symfony\Component\HttpFoundation\Session\Session;

/**
 * Provides base functions for Homegear stories
 *
 */
class HomegearBuilding
{
    const componentBuildings = 'Buildings';
    const componentBuildingGridSort = 'sortBuildingGrid';
    public $hg;
    public $defaultLanguage;

    public function __construct($params)
    {
        $this->hg = new \Homegear\Homegear;
        $this->defaultLanguage = $params->get('default_locale');
    }

    /**
     * Load all known buildings
     * 
     * @return array()
     */
    public function loadAll($saveSession = false)
    {
        $session = new Session();
        if ($saveSession === false) {
            $buildings = $session->get('allBuildings');
        } else {
            try {
                $buildings1 = $this->hg->getData($this::componentBuildings);
                if ($buildings1 && is_array($buildings1)) {
                    $this->_convertOld($buildings1);
                    $this->hg->deleteData($this::componentBuildings);
                }
                $buildings = $this->hg->getBuildings();
                $session->set("allBuildings", $buildings);
            } catch(\Homegear\HomegearException $e) {
                return;
            }
        }
        if ($buildings == null || (is_array($buildings) && count($buildings) == 0)) { return; }
        return $buildings;
    }

    public function loadById($id = null, $session = false)
    {
        $allBuildings = $this->loadAll($session);
        if ($id === null || $id == '') { 
            // no id given, load first room
            if (is_array($allStories) && count($allStories) > 0) {
                $id = $allStories[0]['ID'];
            }
        }
        if ($id !== null && $allBuildings !== null) {
            // get the story metadata out of the story-array
            $result = $allBuildings[array_search($id, array_column($allBuildings,'ID'))];
            // load sorting
            $sort = json_decode($this->hg->getData($this::componentBuildingGridSort,"$id"),true);
            // load stories
            foreach ($result['STORIES'] as $k => $v) {
                $key = 0;
                if (is_array($sort) && count($sort) > 0) {
                    $key = array_search($v, $sort);
                }
                $result['STORIES'][$k] = ['sort' => (isset($key) ? $key : 0), 'id' => $v];
            }
            $result['STORIES'] = ((is_array($result['STORIES']) && count($result['STORIES']) > 0) ? array_values($result['STORIES']) : null);
            return $result;
        }
        return;
    }

    public function getCurrent()
    {
        $session = new Session();
        $current = $session->get('currentBuilding');
        return ($current == null ? "0" : $current);
    }

    private function _convertOld($buildings)
    {
        foreach ($buildings as $building) {
            $building = json_decode($building,true);
            $newId = $this->hg->createBuilding($building['TRANSLATIONS']);
            if (array_key_exists('STORIES', $building) && is_array($building['STORIES'])) {
                foreach ($building['STORIES'] as $story) {
                    $this->hg->addStoryToBuilding(intval($newId),intval($story['id']));
                }
            }
        }
    }
}